from utils import descretize_obs
from env import ContinuousBuildingControlEnvironment as BEnv
import pandas as pd
import numpy as np
from tqdm import tqdm
import itertools
import matplotlib.pyplot as plt

if __name__ == '__main__':
    # start environment
    data_file = 'weather_data_2010_to_2018_summer_pandas.csv'
    # Initialize environment and check dimension
    start = 11040.
    end = 17663.
    env = BEnv(data_file,
               start=start,
               end=end,
               C_env=3.1996e6,
               C_air=3.5187e5,
               R_rc=0.00706,
               R_oe=0.02707,
               R_er=0.00369)
    num_states = env.observation_space.shape[0]-1
    num_actions = env.action_space.shape[0]
    upper_bound = env.action_space.high[0]
    lower_bound = env.action_space.low[0]
    Q_sol_space = [0.0, 0.5, 1.0]
    hour_space = [0.0, 1.0]
    T_cor_space = [0.5]
    n_T_air = 10
    n_T_out = 20
    n_Q_int = 2
    T_air_space = np.round(np.linspace(start=0, stop=1, num=n_T_air+1), 2)
    T_out_space = np.round(np.linspace(start=0, stop=1, num=n_T_out+1), 2)
    Q_int_space = np.round(np.linspace(start=0, stop=1, num=n_Q_int), 2)
    np.random.seed(42)

    ALL_POSSIBLE_ACTIONS = np.round(np.linspace(start=lower_bound,
                                                stop=upper_bound, num=101), 2)

    def epsilon_greedy(Q, s, eps=0.1,
                       ALL_POSSIBLE_ACTIONS=ALL_POSSIBLE_ACTIONS):
        p = np.random.uniform(low=0.0, high=1.0)
        # p = np.random.random()
        if p < (1 - eps):
            # greedy
            argmax_a = Q.loc[[tuple(s)]].iloc[0, :].idxmax()
            return argmax_a
        else:
            return np.random.choice(ALL_POSSIBLE_ACTIONS)

    temp = itertools.product(T_air_space, T_cor_space, T_out_space,
                             Q_sol_space, Q_int_space, Q_sol_space)
    ALL_STATES = list(temp)
    Q_ql = pd.DataFrame(0, columns=ALL_POSSIBLE_ACTIONS,
                        index=ALL_STATES)

    gamma = 0.9
    alpha = 0.1
    n_episodes = 1000
    reward_per_episode = []

    for it in tqdm(range(n_episodes)):
        # begin a new episode
        o, ep_ret, ep_len = env.reset(), 0, 0
        delta = 0
        o = descretize_obs(o)
        done = False
        a = epsilon_greedy(Q_ql, o[1:])
        # keep track of the reward of episode
        episode_reward = 0
        while not done:
            o2, r, done, _ = env.step(a)
            o2 = descretize_obs(o2)
            # update reward
            episode_reward += r
            # get next action
            a2 = epsilon_greedy(Q_ql, o2[1:])
            # update Q(s,a)
            q_old = Q_ql.loc[[tuple(o[1:])], a].iloc[0]
            q_next = gamma*Q_ql.loc[[tuple(o2[1:])], a2].iloc[0]
            q_new = q_old + alpha*(r + q_next - q_old)
            Q_ql.loc[[tuple(o[1:])], a] = q_new
            # next state becomes current state
            a = a2
            o = o2
        if it % 100 == 0:
            print("it: ", it, " with reward: ", episode_reward)
        reward_per_episode.append(episode_reward)

    plt.title("Q-learning reward per episode")
    plt.plot(reward_per_episode)
    plt.xlabel("Iterations")
    plt.ylabel("Reward")
    plt.show()
    # Output Q table
    Q_ql.to_csv("Q_lr.csv")
