from utils import custom_plot
from env import ContinuousBuildingControlEnvironment as BEnv
import pandas as pd


def PID(env, online_start, online_end, title='PID', T_sp=23, Kp=7, Ki=1,
        Kd=0.1):
    o = env.reset()
    episodic_reward = 0
    state_list = []
    action_list = []
    reward_list = []
    energy_list = []
    u_t_list = []
    penalty_list = []
    dt = 1  # each time step dt: 1800 sec = 30 min
    e_p = 0  # placeholder for error
    integral = 0  # placeholder for integral
    while True:
        # perfrom PID
        error = T_sp - (o[1] * 13 + 15)
        integral = integral + error * dt
        derivative = (error-e_p) / dt
        action = Kp * error + Ki * integral + Kd * derivative
        # print(Kp * error , Ki * integral , Kd * derivative)
        e_p = error
        a = action
        # action = policy(tf_prev_state, actor_model, ou_noise)
        # Recieve state and reward from environment.
        o2, r, done, info = env.step(a)
        state_list.append(o[1:])
        action_list.append(a)
        reward_list.append(r)
        u_t_list.append(info["u_t"])
        energy_list.append(info["Energy"])
        penalty_list.append(info["Penalty"])
        episodic_reward += r
        if done:
            break
        o = o2
    df = pd.DataFrame(state_list,
                      columns=['T_air', 'T_cor',
                               'T_out', 'Q_SG', 'Q_int', 'Hour'], dtype=float)
    print("Validate Episode Reward is ==> {}".format(episodic_reward))
    T_air = df['T_air'][-400:]*(28-15)+15
    T_out = df['T_out'][-400:]*(80)-40
    Q_SG = df['Q_SG'][-400:]*1100.
    hour = df['Hour'][-400:]*23
    custom_plot(T_air, hour, T_out, Q_SG, action_list[-400:], u_t_list[-400:],
                title)
    return energy_list, penalty_list


def fix_sp(env, online_start, online_end, title='fix_SP', T_sp=23, db=1.0):
    o = env.reset()
    episodic_reward = 0
    state_list = []
    action_list = []
    reward_list = []
    energy_list = []
    u_t_list = []
    penalty_list = []
    while True:
        a = (T_sp-18)/10
        # action = policy(tf_prev_state, actor_model, ou_noise)
        # Recieve state and reward from environment.
        o2, r, done, info = env.step(a)
        state_list.append(o[1:])
        action_list.append(a)
        reward_list.append(r)
        u_t_list.append(info["u_t"])
        energy_list.append(info["Energy"])
        penalty_list.append(info["Penalty"])
        episodic_reward += r
        if done:
            break
        o = o2
    df = pd.DataFrame(state_list,
                      columns=['T_air', 'T_cor',
                               'T_out', 'Q_SG', 'Q_int', 'Hour'], dtype=float)
    print("Validate Episode Reward is ==> {}".format(episodic_reward))
    T_air = df['T_air'][-400:]*(28-15)+15
    T_out = df['T_out'][-400:]*(80)-40
    Q_SG = df['Q_SG'][-400:]*1100.
    hour = df['Hour'][-400:]*23
    custom_plot(T_air, hour, T_out, Q_SG, action_list[-400:], u_t_list[-400:],
                title)
    return energy_list, penalty_list


if __name__ == '__main__':
    # start environment
    data_file = 'weather_data_2010_to_2018_summer_pandas.csv'
    # Initialize environment and check dimension
    online_start = 17664.
    online_end = 19872.5
    env = BEnv(data_file,
               start=online_start,
               end=online_end,
               C_env=3.1996e6,
               C_air=3.5187e5,
               R_rc=0.00706,
               R_oe=0.02707,
               R_er=0.00369)
    energy_list, penalty_list = fix_sp(env, online_start, online_end)
    print("The energy consumption per day is ",
          sum(energy_list)/len(energy_list)*48*1000)
    print("The temperature violation is per day is ",
          sum(penalty_list)/len(penalty_list)*48*1000)
