import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import matplotlib

n_T_air = 10
n_T_out = 20
n_Q_int = 2


def discrete_Qsol(Q_sol):
    x = 0
    if (Q_sol)*1100 < 500:
        x = 0
    elif ((Q_sol)*1100 >= 500) and ((Q_sol)*1100 < 1000):
        x = 0.5
    else:
        x = 1
    return x


def discrete_hour(hour):
    t = hour*23
    occ = 0
    if (t >= 7) and (t <= 20):
        occ = 1
    return occ


def discrete(x, num):
    # General discrete function
    # x is a 0-1 continuous value
    # num is number of discrete bin
    return int(x / (1/num))/num


def descretize_obs(o, n_T_air=n_T_air, n_T_out=n_T_out, n_Q_int=n_Q_int):
    T_env = o[0]
    T_air = discrete(o[1], num=n_T_air)
    T_cor = 0.5  # discrete(o[2], num=1)
    T_out = discrete(o[3], num=n_T_out)
    Q_SG = discrete_Qsol(o[4])
    Q_int = discrete(o[5], num=n_Q_int)
    hour = discrete_hour(o[6])
    return np.array([T_env, T_air, T_cor, T_out, Q_SG, Q_int, hour])


def custom_plot(T_air, time, T_out, Q_SG, action_list, u_t_list, title):

    sns.set_style("whitegrid")
    font = 15
    matplotlib.rc('xtick', labelsize=font)
    matplotlib.rc('ytick', labelsize=font)
    # scale action back to real value
    action_list = (np.array(action_list)-0.5)*10+23
    lb_list = []
    ub_list = []
    for h in time:
        t = h % 24
        if t >= 7 and t <= 20:
            lb = 22.
            ub = 24.
        else:
            lb = 15
            ub = 28
        lb_list.append(lb)
        ub_list.append(ub)
    fig, axs = plt.subplots(2, 2, figsize=(14, 9), dpi=300)
    time = np.linspace(0, len(time), len(time))
    axs[0, 0].plot(time, action_list, color='maroon', alpha=0.2, label='T_sp')
    axs[0, 0].plot(time, T_air, label='T_air')
    axs[0, 0].plot(time, lb_list, color='orange',
                   label='lower and upper bound')
    axs[0, 0].plot(time, ub_list, color='orange')
    axs[0, 1].plot(time, u_t_list)
    # axs[0, 1].plot(time, actionh_list)
    axs[1, 0].plot(time, T_out)
    axs[1, 1].plot(time, Q_SG)
    axs[0, 0].set_ylim([12, 30])
    axs[0, 1].set_ylim([-1600, 1600])
    axs[1, 0].set_ylim([-30, 40])
    axs[1, 1].set_ylim([0, 1100]) 
    axs[0, 0].set_ylabel("Indoor Air Temperature ($^\circ$C)", fontsize=font)
    axs[0, 1].set_ylabel("Thermal Input (W)", fontsize=font)
    axs[1, 0].set_ylabel("Outdoor Air Temperature ($^\circ$C)", fontsize=font)
    axs[1, 1].set_ylabel("Solar Heat Gain (W)", fontsize=font)
    axs[0, 0].set_xlabel("Time (hr)", fontsize=font)
    axs[0, 1].set_xlabel("Time (hr)", fontsize=font)
    axs[1, 0].set_xlabel("Time (hr)", fontsize=font)
    axs[1, 1].set_xlabel("Time (hr)", fontsize=font)
    axs[0, 0].legend(loc='best')
    fig.suptitle(title, fontsize=font)
    fig.savefig(title+".png")
    plt.close(fig)
    return


def validatepolicy(env, Q, online_start, online_end, title='SARSA'):
    o = env.reset()
    episodic_reward = 0
    state_list = []
    action_list = []
    reward_list = []
    energy_list = []
    u_t_list = []
    penalty_list = []
    while True:
        # perfrom SARSA
        s = descretize_obs(o)[1:]
        a = Q.loc[[tuple(s)]].iloc[0, :].idxmax()
        # action = policy(tf_prev_state, actor_model, ou_noise)
        # Recieve state and reward from environment.
        o2, r, done, info = env.step(a)
        state_list.append(o[1:])
        action_list.append(a)
        reward_list.append(r)
        u_t_list.append(info["u_t"])
        energy_list.append(info["Energy"])
        penalty_list.append(info["Penalty"])
        episodic_reward += r
        if done:
            break
        o = o2
    df = pd.DataFrame(state_list,
                      columns=['T_air', 'T_cor',
                               'T_out', 'Q_SG', 'Q_int', 'Hour'], dtype=float)
    print("Validate Episode Reward is ==> {}".format(episodic_reward))
    T_air = df['T_air'][-400:]*(28-15)+15
    T_out = df['T_out'][-400:]*(80)-40
    Q_SG = df['Q_SG'][-400:]*1100.
    hour = df['Hour'][-400:]*23
    custom_plot(T_air, hour, T_out, Q_SG, action_list[-400:], u_t_list[-400:],
                title)
    return energy_list, penalty_list
